#!/usr/bin/python3

import os,sys,math
import subprocess

###############
##Subroutines##
###############

def callproc(cmd, **kwargs):

    if(sys.version_info > (3,0)):
        cmd = cmd.encode()

    proc = subprocess.Popen(cmd,stderr = subprocess.STDOUT, stdout=subprocess.PIPE, shell=True);
    (out, err) = proc.communicate()
    
    if(sys.version_info > (3,0)):
        out = out.decode()
    
    return out

#Update the timestamp of all created files to match that of the source file
#This should cause less confusion about when things were scanned.
def timestampupdate(fnsource, fnupdate):
    timestamp = os.path.getmtime(fnsource)
    eps = 0.05 #increment for each page number so they are in order by timestamp
    #N = getfnamenum(fnupdate)
    #N = float(N)
    #os.utime(fnupdate,(timestamp+N*eps,timestamp+N*eps))
    os.utime(fnupdate,(timestamp,timestamp))


def makepdffragment(flst,fnfrag):

    fstr = ""
    for f in flst:
        fstr = fstr + f + " "
    cmd = "convert {} {}".format(fstr,fnfrag)
    print("\t{}".format(cmd))
    out = callproc(cmd)
    print("\t{}".format(out))

    return

def pdfunite(fraglst,fnout):

    fstr = ""
    for f in fraglst:
        fstr = fstr + f + " "
    
    cmd = "pdfunite {} {}".format(fstr,fnout)
    print("\t{}".format(cmd))
    out = callproc(cmd)
    print("\t{}".format(out))

    return

################
##Main Routine##
################

if(__name__=="__main__"):

    args = sys.argv

    if(len(args)<2):
        print("ams_bindbook.py: Uses imagemagick and pdfunite to turn directories of images into pdfs.")
        print("Use: ams_bindbook.py [dirname]")
        exit()
    
    dn = args[1]
    if(not os.path.isdir(dn)):
        print("Error: {} doesn't appear to be a directory.".format(dn))
        exit()
    
    fl1 = os.listdir(dn)
    fl2 = []
    for f in fl1:
        f2 = os.path.splitext(f)[0]
        f2e = os.path.splitext(f)[1]

        if(f2e=='.jpg' or f2e=='.JPG'):
            f3 = os.path.join(dn,f)
            fl2.append(f3)
        
    fl2.sort() #sort filenames alphanumerically (page number prefix should provide order)

    if(len(fl2)<=0):
        print("Error, no jpeg files found in directory {}.".format(dn))

    #convert every (stride) jpegs to a pdf fragment
    stride = 10
    I1 = 0
    I2 = min([I1+stride,len(fl2)])
    fragmentlist = []
    K = 0
    while(I1<len(fl2)):

        fnfrag = os.path.join(dn,"{:03d}_{}.pdf".format(K,dn))
        makepdffragment(fl2[I1:I2],fnfrag)
        fragmentlist.append(fnfrag)
        I1 = I1 + stride
        I2 = min([I1+stride,len(fl2)])
        K = K + 1

    #Use pdfunite to concatenate all pdf fragments
    fnout = "{}.pdf".format(dn)
    pdfunite(fragmentlist,fnout)

    #clean up pdf fragments
    for f in fragmentlist:
        os.remove(f)
    
    #update timestamp
    timestampupdate(fl2[0],fnout)






