#!/usr/bin/python3

import os,sys,math
import subprocess

###############
##Subroutines##
###############

def callproc(cmd, **kwargs):

    if(sys.version_info > (3,0)):
        cmd = cmd.encode()

    proc = subprocess.Popen(cmd,stderr = subprocess.STDOUT, stdout=subprocess.PIPE, shell=True);
    (out, err) = proc.communicate()
    
    if(sys.version_info > (3,0)):
        out = out.decode()
    
    return out

def getpdfnpages(fname):
    N = 0
    out = callproc("pdfinfo {} | grep Pages".format(fname))
    out2 = out.split(":")
    if(len(out2)>0):
        out3 = out2[1]
        try:
            N = float(out3)
            N = int(N)
        except:
            N = 0
    
    return N

def dumppage(fname,pgnum,fnamepage):

    fnwoext = os.path.splitext(fname)[0]
    pnwoext = os.path.splitext(fnamepage)[0]
    cmd = "convert -density 220 -quality 80 {}.pdf[{}] {}.jpg".format(fnwoext,pgnum+1,pnwoext)
    out = callproc(cmd)
    print(out)

def pagename(fname,pagenum):
    fnwoext = os.path.splitext(fname)[0]
    s = "{:04d}_{}.jpg".format(pagenum,fnwoext)
    return s


def getfnamenum(pagename):
    N = 0
    qq = os.path.split(pagename)
    q = qq[len(qq)-1]
    q = q.split("_")[0]
    try:
        N = float(q)
        N = int(q)
    except:
        N = 0
    return N

#Update the timestamp of all created files to match that of the source file
#This should cause less confusion about when things were scanned.
def timestampupdate(fname, fnamepage):
    timestamp = os.path.getmtime(fname)
    eps = 0.05 #increment for each page number so they are in order by timestamp
    N = getfnamenum(fnamepage)
    N = float(N)
    os.utime(fnamepage,(timestamp+N*eps,timestamp+N*eps))

###############
##Main Script##
###############

if(__name__=="__main__"):

    args = sys.argv

    if(len(args)<2):
        print("ams_dumplargepdf.py: A wrapper for imagemagick to convert large pdfs one page at a time.")
        print("Converts pdf pages to jpg. It calls the command:")
        print(">convert -density 240 -quality 80 -resize 80% [fname][pgnum]  [fname].jpg")
        print("Use: ams_dumplargepdf.py [fname]")
        exit()
    
    fname = args[1]

    N = getpdfnpages(fname)
    pages = []

    if(N<=0):
        print("Error: pdfinfo returned {} pages for {}".format(N,fname))
        exit()
    
    dn = os.path.splitext(fname)[0]
    if(not os.path.isdir(dn)):
        os.mkdir(dn)
    if(not os.path.isdir(dn)):
        print("Error: Could not create directory {} for page output.".format(dn))
        exit()
    
    for I in range(0,N):
        print("{}: Dumping page {} of {}".format(fname,I,N))
        pn = pagename(fname,I)
        pn2 = os.path.join(dn,pn)
        dumppage(fname,I,pn2)
        timestampupdate(fname,pn2)
    
    #fname = "camping_and_woodcraft_vol2.pdf"
    
